@@ -11,6 +11,7 @@ from .result import *
 from .statement import *
 from .subscription import *
 from .schemaless import *
+
 try:
     from .sqlalchemy import *
 except:
@@ -20,10 +21,13 @@ from taos._version import __version__
 
 # Globals
 threadsafety = 0
+"""sqlalchemy will read this attribute"""
 paramstyle = "pyformat"
+"""sqlalchemy will read this attribute"""
 
 __all__ = [
     "__version__",
+    "IS_V3",
     # functions
     "connect",
     "new_bind_param",
@@ -43,6 +47,7 @@ __all__ = [
     "SmlProtocol"
 ]
 
+
 def connect(*args, **kwargs):
     # type: (..., ...) -> TaosConnection
     """Function to return a TDengine connector object
