@@ -1,5 +1,7 @@
 # encoding:UTF-8
 import ctypes
+
+from .cinterface import IS_V3
 from .constants import FieldType
 from .error import *
 from .precision import *
@@ -10,7 +12,8 @@ import sys
 _datetime_epoch = datetime.utcfromtimestamp(0)
 
 def _is_not_none(obj):
-    obj != None
+    return obj != None
+
 class TaosBind(ctypes.Structure):
     _fields_ = [
         ("buffer_type", c_int),
@@ -24,124 +27,180 @@ class TaosBind(ctypes.Structure):
         ("allocated", c_int),
     ]
 
-    def null(self):
-        self.buffer_type = FieldType.C_NULL
-        self.is_null = pointer(c_int(1))
-
     def bool(self, value):
         self.buffer_type = FieldType.C_BOOL
-        self.buffer = cast(pointer(c_bool(value)), c_void_p)
-        self.buffer_length = sizeof(c_bool)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_bool(value)), c_void_p)
+            self.buffer_length = sizeof(c_bool)
 
     def tinyint(self, value):
         self.buffer_type = FieldType.C_TINYINT
-        self.buffer = cast(pointer(c_int8(value)), c_void_p)
-        self.buffer_length = sizeof(c_int8)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_int8(value)), c_void_p)
+            self.buffer_length = sizeof(c_int8)
 
     def smallint(self, value):
         self.buffer_type = FieldType.C_SMALLINT
-        self.buffer = cast(pointer(c_int16(value)), c_void_p)
-        self.buffer_length = sizeof(c_int16)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_int16(value)), c_void_p)
+            self.buffer_length = sizeof(c_int16)
 
     def int(self, value):
         self.buffer_type = FieldType.C_INT
-        self.buffer = cast(pointer(c_int32(value)), c_void_p)
-        self.buffer_length = sizeof(c_int32)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_int32(value)), c_void_p)
+            self.buffer_length = sizeof(c_int32)
 
     def bigint(self, value):
         self.buffer_type = FieldType.C_BIGINT
-        self.buffer = cast(pointer(c_int64(value)), c_void_p)
-        self.buffer_length = sizeof(c_int64)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_int64(value)), c_void_p)
+            self.buffer_length = sizeof(c_int64)
 
     def float(self, value):
         self.buffer_type = FieldType.C_FLOAT
-        self.buffer = cast(pointer(c_float(value)), c_void_p)
-        self.buffer_length = sizeof(c_float)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_float(value)), c_void_p)
+            self.buffer_length = sizeof(c_float)
 
     def double(self, value):
         self.buffer_type = FieldType.C_DOUBLE
-        self.buffer = cast(pointer(c_double(value)), c_void_p)
-        self.buffer_length = sizeof(c_double)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_double(value)), c_void_p)
+            self.buffer_length = sizeof(c_double)
 
     def binary(self, value):
         buffer = None
         length = 0
-        if isinstance(value, str):
-            bytes = value.encode("utf-8")
-            buffer = create_string_buffer(bytes)
-            length = len(bytes)
-        else:
-            buffer = value
-            length = len(value)
         self.buffer_type = FieldType.C_BINARY
-        self.buffer = cast(buffer, c_void_p)
-        self.buffer_length = length
-        self.length = pointer(c_size_t(self.buffer_length))
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            if isinstance(value, str):
+                bytes = value.encode("utf-8")
+                buffer = create_string_buffer(bytes)
+                length = len(bytes)
+            else:
+                buffer = value
+                length = len(value)
+            self.buffer = cast(buffer, c_void_p)
+            self.buffer_length = length
+            self.length = pointer(c_size_t(self.buffer_length))
 
     def timestamp(self, value, precision=PrecisionEnum.Milliseconds):
-        if type(value) is datetime:
-            if precision == PrecisionEnum.Milliseconds:
-                ts = int(round((value - _datetime_epoch).total_seconds() * 1000))
-            elif precision == PrecisionEnum.Microseconds:
-                ts = int(round((value - _datetime_epoch).total_seconds() * 10000000))
-            else:
-                raise PrecisionError("datetime do not support nanosecond precision")
-        elif type(value) is float:
-            if precision == PrecisionEnum.Milliseconds:
-                ts = int(round(value * 1000))
-            elif precision == PrecisionEnum.Microseconds:
-                ts = int(round(value * 10000000))
-            else:
-                raise PrecisionError("time float do not support nanosecond precision")
-        elif isinstance(value, int) and not isinstance(value, bool):
-            ts = value
-        elif isinstance(value, str):
-            value = datetime.fromisoformat(value)
-            if precision == PrecisionEnum.Milliseconds:
-                ts = int(round(value * 1000))
-            elif precision == PrecisionEnum.Microseconds:
-                ts = int(round(value * 10000000))
-            else:
-                raise PrecisionError("datetime do not support nanosecond precision")
-
         self.buffer_type = FieldType.C_TIMESTAMP
-        self.buffer = cast(pointer(c_int64(ts)), c_void_p)
-        self.buffer_length = sizeof(c_int64)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            if type(value) is datetime:
+                if precision == PrecisionEnum.Milliseconds:
+                    ts = int(round((value - _datetime_epoch).total_seconds() * 1000))
+                elif precision == PrecisionEnum.Microseconds:
+                    ts = int(round((value - _datetime_epoch).total_seconds() * 10000000))
+                else:
+                    raise PrecisionError("datetime do not support nanosecond precision")
+            elif type(value) is float:
+                if precision == PrecisionEnum.Milliseconds:
+                    ts = int(round(value * 1000))
+                elif precision == PrecisionEnum.Microseconds:
+                    ts = int(round(value * 10000000))
+                else:
+                    raise PrecisionError("time float do not support nanosecond precision")
+            elif isinstance(value, int) and not isinstance(value, bool):
+                ts = value
+            elif isinstance(value, str):
+                value = datetime.fromisoformat(value)
+                if precision == PrecisionEnum.Milliseconds:
+                    ts = int(round(value * 1000))
+                elif precision == PrecisionEnum.Microseconds:
+                    ts = int(round(value * 10000000))
+                else:
+                    raise PrecisionError("datetime do not support nanosecond precision")
+            self.buffer = cast(pointer(c_int64(ts)), c_void_p)
+            self.buffer_length = sizeof(c_int64)
 
     def nchar(self, value):
         buffer = None
         length = 0
-        if isinstance(value, str):
-            bytes = value.encode("utf-8")
-            buffer = create_string_buffer(bytes)
-            length = len(bytes)
-        else:
-            buffer = value
-            length = len(value)
         self.buffer_type = FieldType.C_NCHAR
-        self.buffer = cast(buffer, c_void_p)
-        self.buffer_length = length
-        self.length = pointer(c_size_t(self.buffer_length))
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            if isinstance(value, str):
+                bytes = value.encode("utf-8")
+                buffer = create_string_buffer(bytes)
+                length = len(bytes)
+            else:
+                buffer = value
+                length = len(value)
+            self.buffer = cast(buffer, c_void_p)
+            self.buffer_length = length
+            self.length = pointer(c_size_t(self.buffer_length))
+
+    def json(self, value):
+        buffer = None
+        length = 0
+        self.buffer_type = FieldType.C_JSON
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            if isinstance(value, str):
+                bytes = value.encode("utf-8")
+                buffer = create_string_buffer(bytes)
+                length = len(bytes)
+            else:
+                buffer = value
+                length = len(value)
+            self.buffer = cast(buffer, c_void_p)
+            self.buffer_length = length
+            self.length = pointer(c_size_t(self.buffer_length))
 
     def tinyint_unsigned(self, value):
         self.buffer_type = FieldType.C_TINYINT_UNSIGNED
-        self.buffer = cast(pointer(c_uint8(value)), c_void_p)
-        self.buffer_length = sizeof(c_uint8)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_uint8(value)), c_void_p)
+            self.buffer_length = sizeof(c_uint8)
 
     def smallint_unsigned(self, value):
         self.buffer_type = FieldType.C_SMALLINT_UNSIGNED
-        self.buffer = cast(pointer(c_uint16(value)), c_void_p)
-        self.buffer_length = sizeof(c_uint16)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_uint16(value)), c_void_p)
+            self.buffer_length = sizeof(c_uint16)
 
     def int_unsigned(self, value):
         self.buffer_type = FieldType.C_INT_UNSIGNED
-        self.buffer = cast(pointer(c_uint32(value)), c_void_p)
-        self.buffer_length = sizeof(c_uint32)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_uint32(value)), c_void_p)
+            self.buffer_length = sizeof(c_uint32)
+        
 
     def bigint_unsigned(self, value):
         self.buffer_type = FieldType.C_BIGINT_UNSIGNED
-        self.buffer = cast(pointer(c_uint64(value)), c_void_p)
-        self.buffer_length = sizeof(c_uint64)
+        if value is None:
+            self.is_null = pointer(c_int(1))
+        else:
+            self.buffer = cast(pointer(c_uint64(value)), c_void_p)
+            self.buffer_length = sizeof(c_uint64)
 
 
 def _datetime_to_timestamp(value, precision):
@@ -187,13 +246,9 @@ class TaosMultiBind(ctypes.Structure):
         ("num", c_int),
     ]
 
-    def null(self, num):
-        self.buffer_type = FieldType.C_NULL
-        self.is_null = cast((c_char * num)(*[1 for _ in range(num)]), c_char_p)
-        self.buffer = c_void_p(None)
-        self.num = num
-
     def bool(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         try:
             buffer = cast(values, c_void_p)
         except:
@@ -207,11 +262,13 @@ class TaosMultiBind(ctypes.Structure):
         self.num = len(values)
         self.buffer_type = FieldType.C_BOOL
         self.buffer_length = sizeof(c_bool)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def tinyint(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_TINYINT
         self.buffer_length = sizeof(c_int8)
-
         try:
             buffer = cast(values, c_void_p)
         except:
@@ -220,11 +277,13 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*values)
             except:
                 buffer = buffer_type(*[v if v is not None else FieldType.C_TINYINT_NULL for v in values])
-
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def smallint(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_SMALLINT
         self.buffer_length = sizeof(c_int16)
 
@@ -238,8 +297,11 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_SMALLINT_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def int(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_INT
         self.buffer_length = sizeof(c_int32)
 
@@ -253,8 +315,11 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_INT_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def bigint(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_BIGINT
         self.buffer_length = sizeof(c_int64)
 
@@ -268,8 +333,11 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_BIGINT_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def float(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_FLOAT
         self.buffer_length = sizeof(c_float)
 
@@ -283,8 +351,11 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_FLOAT_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def double(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_DOUBLE
         self.buffer_length = sizeof(c_double)
 
@@ -298,28 +369,16 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_DOUBLE_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
-    def binary(self, values):
+    def _str_to_buffer(self, values):
         self.num = len(values)
-        self.buffer = cast(c_char_p("".join(filter(_is_not_none, values)).encode("utf-8")), c_void_p)
-        self.length = (c_int * len(values))(*[len(value) if value is not None else 0 for value in values])
-        self.buffer_type = FieldType.C_BINARY
-        self.is_null = cast((c_byte * self.num)(*[1 if v == None else 0 for v in values]), c_char_p)
-
-    def timestamp(self, values, precision=PrecisionEnum.Milliseconds):
-        try:
-            buffer = cast(values, c_void_p)
-        except:
-            buffer_type = c_int64 * len(values)
-            buffer = buffer_type(*[_datetime_to_timestamp(value, precision) for value in values])
-
-        self.buffer_type = FieldType.C_TIMESTAMP
-        self.buffer = cast(buffer, c_void_p)
-        self.buffer_length = sizeof(c_int64)
-        self.num = len(values)
-
-    def nchar(self, values):
-        # type: (list[str]) -> None
+        is_null = [1 if v is None else 0 for v in values]
+        self.is_null = cast((c_byte * self.num)(*is_null), c_char_p)
+        
+        if sum(is_null) == self.num:
+            self.length = (c_int32 * len(values))(0 * self.num)
+            return
         if sys.version_info < (3, 0):
             _bytes = [bytes(value) if value is not None else None for value in values]
             buffer_length = max(len(b) + 1 for b in _bytes if b is not None)
@@ -347,11 +406,45 @@ class TaosMultiBind(ctypes.Structure):
             )
         self.length = (c_int32 * len(values))(*[len(b) if b is not None else 0 for b in _bytes])
         self.buffer_length = buffer_length
+        
+    def binary(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
+        self.buffer_type = FieldType.C_BINARY
+        self._str_to_buffer(values)
+
+    def timestamp(self, values, precision=PrecisionEnum.Milliseconds):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
+        try:
+            buffer = cast(values, c_void_p)
+        except:
+            buffer_type = c_int64 * len(values)
+            buffer = buffer_type(*[_datetime_to_timestamp(value, precision) for value in values])
+
+        self.buffer_type = FieldType.C_TIMESTAMP
+        self.buffer = cast(buffer, c_void_p)
+        self.buffer_length = sizeof(c_int64)
         self.num = len(values)
-        self.is_null = cast((c_byte * self.num)(*[1 if v == None else 0 for v in values]), c_char_p)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
+
+    def nchar(self, values):
+        # type: (list[str]) -> None
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_NCHAR
+        self._str_to_buffer(values)
+
+    def json(self, values):
+        # type: (list[str]) -> None
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
+        self.buffer_type = FieldType.C_JSON
+        self._str_to_buffer(values)
 
     def tinyint_unsigned(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_TINYINT_UNSIGNED
         self.buffer_length = sizeof(c_uint8)
 
@@ -365,8 +458,11 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_TINYINT_UNSIGNED_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def smallint_unsigned(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_SMALLINT_UNSIGNED
         self.buffer_length = sizeof(c_uint16)
 
@@ -380,8 +476,11 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_SMALLINT_UNSIGNED_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def int_unsigned(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_INT_UNSIGNED
         self.buffer_length = sizeof(c_uint32)
 
@@ -395,8 +494,11 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_INT_UNSIGNED_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
     def bigint_unsigned(self, values):
+        if type(values) is not tuple and type(values) is not list:
+            values = tuple([values])
         self.buffer_type = FieldType.C_BIGINT_UNSIGNED
         self.buffer_length = sizeof(c_uint64)
 
@@ -410,16 +512,23 @@ class TaosMultiBind(ctypes.Structure):
                 buffer = buffer_type(*[v if v is not None else FieldType.C_BIGINT_UNSIGNED_NULL for v in values])
         self.buffer = cast(buffer, c_void_p)
         self.num = len(values)
+        self.is_null = cast((c_char * len(values))(*[1 if value is None else 0 for value in values]), c_char_p)
 
 
 def new_bind_param():
     # type: () -> TaosBind
-    return TaosBind()
+    if IS_V3:
+        return TaosMultiBind()
+    else:
+        return TaosBind()
 
 
 def new_bind_params(size):
     # type: (int) -> Array[TaosBind]
-    return (TaosBind * size)()
+    if IS_V3:
+        return (TaosMultiBind * size)()
+    else:
+        return (TaosBind * size)()
 
 
 def new_multi_bind():
