@@ -1,5 +1,4 @@
 from typing import List, Dict
-
 from .errors import NotSupportedError
 from .cursor import TaosRestCursor
 from .restclient import RestClient
@@ -7,7 +6,10 @@ from .restclient import RestClient
 
 class Result:
     def __init__(self, resp: dict):
-        self.code: str = resp["code"]
+        if "status" in resp:
+            self.status: str = resp["status"]
+        else:
+            self.code: str = resp["code"]
         self.column_meta: List[list] = resp["column_meta"]
         self.data: List[list] = resp["data"]
         self.rows: int = resp["rows"]
