@@ -93,8 +93,12 @@ class RestClient:
         self._check_status(response)
 
         resp = json.load(response)
-        if resp["code"] != 0:
-            raise ConnectError(resp["desc"], resp["code"])
+        if "status" in resp:
+            if resp["status"] != "succ":
+                raise ConnectError(resp["desc"], status=resp["status"])
+        else:
+            if resp["code"] != 0:
+                raise ConnectError(resp["desc"], resp["code"])
         return resp["desc"]
 
     def sql(self, q: str) -> dict:
@@ -112,8 +116,13 @@ class RestClient:
         response = urlopen(request, timeout=self._timeout)
         self._check_status(response)
         resp = json.load(response)
-        if resp["code"] != 0:
-            raise ExecutionError(resp["desc"], resp["code"])
+        if "status" in resp:
+            # v2
+            if resp["status"] != "succ":
+                raise ConnectError(resp["desc"], status=resp["status"])
+        else:
+            if resp["code"] != 0:
+                raise ConnectError(resp["desc"], resp["code"])
         if self._convert_timestamp:
             self._convert_time(resp)
         return resp
@@ -126,7 +135,7 @@ class RestClient:
         data = resp["data"]
         ts_cols = []
         for i in range(len(meta)):
-            if meta[i][1] == "TIMESTAMP":
+            if meta[i][1] == "TIMESTAMP" or meta[i][1] == 9:
                 ts_cols.append(i)
         if len(ts_cols) == 0:
             return
